import fileinput
import os
import re
import sys

from django.conf import settings
from django.utils.crypto import get_random_string


def generate_secret_key(length=50):
    """
        Return a 50 character random string usable as a SECRET_KEY setting value.
    """
    try:
        length = int(length)
    except ValueError:
        length = 50
    except TypeError:
        length = 50

    chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'

    return get_random_string(length, chars)


def save_to_settings(value, parameter_name, settings_dir_name=''):
    """
        Save the value to the given parameter in settings.py file.
    """
    settings_dir = os.path.join(settings.BASE_DIR, settings_dir_name, 'settings.py')

    if not os.path.exists(settings_dir):
        return False, f'Not found: {settings_dir}\n\nCan\'t find \'settings.py\' file,' \
            ' please specify settings file directory name using -p argument.'

    result, message = _replace_line(value, parameter_name, settings_dir)

    return result, message


def _replace_line(value, parameter_name, settings_file):
    parameter_is_exist = False

    if not parameter_name:
        return False, f'Can\'t find passed parameter name. (parameter: {parameter_name})'

    new_line = f'{parameter_name} = {value}'
    line_pattern = fr'^{parameter_name} = .*'

    for line in fileinput.input(settings_file, inplace=True):
        if re.match(line_pattern, line):
            parameter_is_exist = True
            line = re.sub(line_pattern, new_line, line)

        sys.stdout.write(line)

    if not parameter_is_exist:
        return False, f'Can\'t find passed parameter name. (parameter: {parameter_name})'

    return True, f'{parameter_name} updated successfully.'



from django.core.management import BaseCommand

from djecrety.utils import generate_secret_key, save_to_settings


class Command(BaseCommand):
    help = 'Generate a new secret key with 50 characters length.'

    def add_arguments(self, parser):
        parser.add_argument('-s', '--save', action='store_true',
                            help='Save the generated secret key to settings.py file.')
        parser.add_argument('-p', '--print', action='store_true',
                            help='Display the generated secret key when saving on file.')
        parser.add_argument('-d', '--settings-dir-name', type=str,
                            help='Specify settings directory name.')

    def handle(self, *args, **kwargs):
        new_key = '\'{}\''.format(generate_secret_key())

        if kwargs['save']:
            settings_dir_name = kwargs['settings_dir_name']
            result, message = save_to_settings(new_key, 'SECRET_KEY', settings_dir_name=settings_dir_name)

            if result:
                self.stdout.write(self.style.SUCCESS(message))

                if kwargs['print']:
                    self.stdout.write('{} {}'.format(self.style.SUCCESS('SECRET_KEY set to:'), new_key))

                self._print_tips()
            else:
                self.stdout.write(self.style.ERROR(message))
        else:
            self.stdout.write('{} {}'.format(self.style.SUCCESS('New secret key:'), new_key))
            self._print_tips()

    def _print_tips(self):
        self.stdout.write(self.style.WARNING('\nrecommended:'))
        self.stdout.write('\t1. Keep it safe.\n\t2. Ignore \'settings.py\' file in your commits.\n\t'
                          '3. Change the secret key on your deploy.\n\t'
                          '4. If you lost the secret key for any reason or\n\t'
                          '   the server got compromised change it as soon as possible.')



import os

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    long_description = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='djecrety',
    version='1.0.2',
    python_requires='>=3.5',
    packages=find_packages(exclude=['testproject']),
    include_package_data=True,
    license='GNU Version 3 License',
    description='Djecrety package provides a command to generate a new secret key for your project and \
                save it to settings.py file.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://djecrety.ir/',
    author='Majid Rouhi',
    author_email='mrouhi13@gmail.com',
    zip_safe=False,
    install_requires=[
        'Django>=2.0',
        'pytz==2019.1'
    ],
    classifiers=[
        'Environment :: Console',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Security :: Cryptography',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'
    ],
)
